Unit TEvoCreatureListSrc;
Interface
Uses
	Classes,
        TEvoCreatureSrc,
        graphics,
        windows,
        EvoStoreSrc,
	sysutils;

type
TEvoCreatureList = class (tlist)		 {TEvoCreatureList}
  protected
        function FastIndexOfX(AX : Integer) : integer;
        procedure AddNewHatches;
        procedure BreedorLive;

  public
        procedure ClearAll;
	destructor Destroy; override;
	procedure Paint(aCanvas : TCanvas);
        procedure MoveAll;
        procedure Tick;
        Procedure MakeCreature( pCreatureProperties : tCreatureProperties);

end; {TEvoCreatureList}

Var
	EvoCreatureList : TEvoCreatureList;

implementation

destructor TEvoCreatureList.Destroy;

begin
   clearAll;
   inherited destroy;
end;

Procedure TEvoCreatureList.MakeCreature( pCreatureProperties : tCreatureProperties);
var
	EvoCreature : TEvoCreature;

begin
     EvoCreature := TEvoCreature.create;
     With EvoCreature do SetFromRecord(pCreatureProperties);
     add(EvoCreature);
     EvoCreature.AddToCensus;
end;

procedure TEvoCreatureList.ClearAll;
var
   EvoCreature : TEvoCreature;
   lc, mc : integer;

begin
   mc := pred(count);
   for lc := 0 to mc do begin
	EvoCreature := TEvoCreature(items[lc]);
        EvoCreature.free;
   end;
   clear;
end;

procedure TEvoCreatureList.Paint(aCanvas : TCanvas);

var
   EvoCreature : TEvoCreature;
   lc, mc : integer;

begin
   mc := pred(count);
   for lc := 0 to mc do begin
       EvoCreature := TEvoCreature(items[lc]);
       EvoCreature.Paint(aCanvas);
   end;
end;

function XSortCompare (Item1, Item2: Pointer): Integer;

var
  aEvoCreature, bEvoCreature : TEvoCreature;

begin
   aEvoCreature := TEvoCreature(item1);
   bEvoCreature := TEvoCreature(item2);
   result := aEvoCreature.LocPoint.x - bEvoCreature.LocPoint.x;
   if (result = 0) then result := aEvoCreature.LocPoint.y - bEvoCreature.LocPoint.y;
end;

procedure TEvoCreatureList.MoveAll;

var
   EvoCreature : TEvoCreature;
   lc, mc : integer;

begin
   mc := pred(count);
   lc := 0;
   while (lc <= mc) do begin
	EvoCreature := TEvoCreature(items[lc]);
        if (EvoCreature.Alive) then begin
              EvoCreature.Step;
              inc(lc);
        end
        else begin
               delete(lc);
               if not(EvoCreature.animalstate in [tadead, taBeingEaten]) then inc(deadcount);
               EvoCreature.animalstate := tadead;
               ReincarnationList.add(EvoCreature);
               dec(mc);
        end;
   end;
   sort(XSortCompare);
end;

procedure TEvoCreatureList.BreedorLive;
Var
   lc2, EncounterRes, lc, mc : integer;
   available, notbreeded, cont, tb1, tb : boolean;
   TargetEvoCreature, EvoCreatureToEat, EvoCreature2, EvoCreature : TEvoCreature;
   infrect : trect;

begin
 mc := pred(count);
 lc := 0;
 while (lc <= mc) do begin
   EvoCreature := TEvoCreature(items[lc]);
   available := EvoCreature.IsAvailable;
   if available then begin
     TargetEvoCreature := nil;
     EvoCreatureToEat := nil;
     nexTEvoCreature := nil;
     EvoCreature.influence(infrect);
     if (infrect.Left > 1) then lc2 := FastIndexOfX(infrect.TopLeft.x-1)
                           else lc2 := 0;
     cont := (lc2 <= mc) and (lc2 <> -1);
     notbreeded := true;
     while (cont and notbreeded) do begin
	EvoCreature2 := TEvoCreature(items[lc2]);
        if ((EvoCreature <> EvoCreature2) and myPtInRect(infrect, EvoCreature2.LocPoint) and EvoCreature2.IsAvailable) then begin
           if EvoCreature.breed( EvoCreature2) then notbreeded := false
           else begin
              EncounterRes := EvoCreature.Encounter( EvoCreature2);
              case EncounterRes of
                  -1 : EvoCreature.Competes(EvoCreature2);
                   0 : {do nothing};
                   1 : EvoCreatureToEat := EvoCreature2;
                else TargetEvoCreature := EvoCreature2;
              end;
           end;
        end;
        inc(lc2);
        cont := (EvoCreature2.locpoint.x <= infrect.right) and (lc2 <= mc);
     end;
     if (notbreeded and available) then begin
            if (EvoCreatureToEat <> nil) then EvoCreature.Eats(EvoCreatureToEat)
            else begin
               if (nexTEvoCreature <> nil) and (EvoCreature.breedptr = nil) then begin
                        EvoCreature.breedptr := nexTEvoCreature;
                        EvoCreature.FoundPartner := true;
               end
               else begin
                  if (EvoCreature.foodptr = nil) and (TargetEvoCreature <> nil) then begin
                    EvoCreature.foodptr := TargetEvoCreature;
                    EvoCreature.foundfood := true;
                  end;
               end;
               EvoCreature.DecideNextHeading;
            end;
     end;
   end;
   if (EvoCreature.animalstate = taEating) and (EvoCreature.EatingCreature.animalstate <> tadead) then begin
      EvoCreature.Eats(EvoCreature.EatingCreature);
   end;
   inc(lc);
 end;
end;

procedure TEvoCreatureList.AddNewHatches;
Var
   rc, lc, mc : integer;
   EvoCreature : TEvoCreature;

begin
   mc := pred(BreadCreatureList.count);
   didnotbreed := ((mc + count) > SaverOptions.MaximumNumberOfCreatures);
   if didnotbreed then begin
     for lc := 0 to mc do begin
          EvoCreature := TEvoCreature(BreadCreatureList[lc]);
          EvoCreature.free;
     end;
   end
   else begin
     for lc := 0 to mc do begin
          EvoCreature := TEvoCreature(BreadCreatureList[lc]);
          add(EvoCreature );
          inc(breadcount);
     end;
   end;
end;

procedure TEvoCreatureList.Tick;

begin
   if stopping then exit;
   BreadCreatureList.clear;
   BreedorLive;
   AddNewHatches;
end;


function TEvoCreatureList.FastIndexOfX(AX : Integer) : integer;
var
   LMI, cres, LeftIndex, RightIndex : integer;
   aEvoCreature : TEvoCreature;

begin
   LeftIndex := 0;
   RightIndex := pred(Count);
   lmi := RightIndex;
   Result := 0;
   while (LeftIndex <= RightIndex) do begin
      Result := (LeftIndex + RightIndex) div 2;
      aEvoCreature := TEvoCreature(items[Result]);
      with aEvoCreature.LocPoint do cres := (AX - X);
      if (cres = 0) then exit
      else begin
         IF (LMI = Result) THEN exit;
         if (cres > 0) then LeftIndex := Result + 1
                       else RightIndex := Result - 1;
      end;
      LMI := Result;
   end;
end;


{ End of code  }
end.

